/******************************************************************************
 *
 * ClassDB - A simple school calendar 
 * Copyright (C) 2001 Ricardo Cabral - nightdog@users.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *
 *****************************************************************************/

#include <PalmOS.h>
#include "ClassDBRsc.h"
#include "Prototypes.h"

/***********************************************************************
 *
 *   Entry Points
 *
 ***********************************************************************/

 

/***********************************************************************
 *
 *   Internal Structures
 *
 ***********************************************************************/



/***********************************************************************
 *
 *   Global variables
 *
 ***********************************************************************/
//static Boolean HideSecretRecords;

Boolean firstDraw = true;
EVENTS year[12][31];
UInt8 currentday, currentmonth;
UInt16 today, thisMonth, thisYear;
Char class[12];
Char *event;
/***********************************************************************
 *
 *   Internal Constants
 *
 ***********************************************************************/
#define appFileCreator				 'CsDB'
#define appVersionNum              0x01
#define appPrefID                  0x00
#define appPrefVersionNum          0x01
#define databaseName				"EvtList"

// classs list popup
#define Portuguese					0
#define English						1
#define Math						2
#define German						3
#define Biology						4
#define French						5
#define Geology						6
#define Geography					7
#define History						8
#define Computers					9
#define Physics						10
#define Chemistry					11
#define Spanish						12


// Define the minimum OS version we support (2.0 for now).
#define ourMinVersion	sysMakeROMVersion(3,2,0,sysROMStageRelease,0)


/***********************************************************************
 *
 *   Internal Functions
 *
 ***********************************************************************/


/***********************************************************************
 *
 * FUNCTION:    RomVersionCompatible
 *
 * DESCRIPTION: This routine checks that a ROM version is meet your
 *              minimum requirement.
 *
 * PARAMETERS:  requiredVersion - minimum rom version required
 *                                (see sysFtrNumROMVersion in SystemMgr.h 
 *                                for format)
 *              launchFlags     - flags that indicate if the application 
 *                                UI is initialized.
 *
 * RETURNED:    error code or zero if rom is compatible
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
	UInt32 romVersion;

	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
		{
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
			{
			FrmAlert (RomIncompatibleAlert);
		
			// Palm OS 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (romVersion < ourMinVersion)
				{
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
				}
			}
		
		return sysErrRomIncompatible;
		}

	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: This routine returns a pointer to an object in the current
 *              form.
 *
 * PARAMETERS:  formId - id of the form to display
 *
 * RETURNED:    void *
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void * GetObjectPtr(UInt16 objectID)
{
	FormPtr frmP;

	frmP = FrmGetActiveForm();
	return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID));
}


#pragma mark ----------------



/***********************************************************************
 *
 * FUNCTION:    AboutFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "AboutForm" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AboutFormHandleEvent(EventPtr eventP)
{
   Boolean handled = false;
   FormPtr frmP;

	switch (eventP->eType) 
		{
		case ctlSelectEvent:
			FrmGotoForm(MainForm);
			break;

		case frmOpenEvent:
			frmP = FrmGetActiveForm();
			// AboutFormInit( frmP);
			FrmDrawForm ( frmP);
			handled = true;
			break;

		default:
			break;
		
		}
	
	return handled;
}


#pragma mark ----------------

/***********************************************************************
 *
 * FUNCTION:    EventEditHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "EventEdit" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean EventEditHandleEvent(EventPtr eventP)
{
   Boolean handled = false;
   FormPtr prevForm;
   UInt16 prevFormID = FrmGetActiveFormID();
   UInt16 returnedButton;
   Char *currentdaystr, *currentmonthstr, s[4];
   MemPtr	dayText, monthText, eventText;
   MemHandle	monthHandle, dayHandle, eventHandle;
   FieldType *currentdayptr, *currentmonthptr, *eventFldPtr;
   Int16 selection;
   Char * label;
   ListPtr lst;
   ControlPtr ctl;

	switch (eventP->eType)
		{
		case ctlSelectEvent:
			switch(eventP->data.ctlEnter.controlID) {
				case EventEditOKButton:
					// get the text in the event field
					event = FldGetTextPtr(GetObjectPtr(EventEditTypeField));
					// save data in record overriding the existing one and redraw screen
					// CHECK IF THERE'S DATA ON all fields
					if(StrCompare(class, "none") == 0)
						FrmAlert (ThereisnodataAlert);
					else if(event == NULL)
						FrmAlert (ThereisnodataAlert);
					else {
						// copy the new values to the main global struct
							StrCopy(year[currentmonth][currentday].class, class);
							StrCopy(year[currentmonth][currentday].event, event); 
						// this tells the main gadget that current record is used and needs to be drawn
							year[currentmonth][currentday].used = true;
						// return to whatever form loaded me
							FrmReturnToForm(0);
						// if that form is the main form, refresh the main gadget
							prevForm = FrmGetActiveForm();
							if(FrmGetActiveFormID() == MainForm)
								GadgetDraw(prevForm, Main1Gadget);				
							
					}
					handled = true;
					break;
				case EventEditDeleteButton:
						returnedButton = FrmAlert (DeleteAlert);
						if(returnedButton == 0) {
						// erase data in record
							year[currentmonth][currentday].used = false;
							FrmReturnToForm(0);
							prevForm = FrmGetActiveForm();
							if(FrmGetActiveFormID() == MainForm)
								GadgetDraw(prevForm, Main1Gadget);	
						}
							
						handled = true;
						break;
				case EventEditCancelButton:
					// ignore the current data, whether it was changed or not and go to the main form
						FrmReturnToForm(0);
						handled = true;
						break;
				default:
					break;
			}
						





		case popSelectEvent:
			switch(eventP->data.popSelect.selection) {
					case Portuguese:
						StrCopy(class, "Portuguese");
						break;
					case Math:
						StrCopy(class,"Math"); 
						break;
					case French:
						StrCopy(class,"French"); 
						break;
					case Geography:
						StrCopy(class,"Geography"); 
						break;
					case Spanish:
						StrCopy(class,"Spanish"); 
						break;	
					case Chemistry:
						StrCopy(class,"Chemistry"); 
						break;	
					case Physics:
						StrCopy(class,"Physics"); 
						break;	
					case Computers:
						StrCopy(class,"Computers"); 
						break;	
					case History:
						StrCopy(class,"History"); 
						break;	
					case German:
						StrCopy(class,"German"); 
						break;	
					case Biology:
						StrCopy(class,"Biology"); 
						break;	
					case English:
						StrCopy(class,"English"); 
						break;	
					case Geology:
						StrCopy(class,"Geology"); 
						break;
					default:
						break;
				}
			break;

		case frmOpenEvent:			
			prevForm = FrmGetActiveForm();
	// we need to update the fields with the information of the day we're editing
	
		// Make the popup menu display the correct class 
			StrCopy(class, year[currentmonth][currentday].class);
			
			
			if(StrCompare(class, "none") != 0) {
				
				if(StrCompare(class, "Portuguese") == 0)
					selection = 0;
					
				if(StrCompare(class, "English") == 0)
					selection = 1;
						
				if(StrCompare(class, "Math") == 0)
					selection = 2;
						
				if(StrCompare(class, "German") == 0)
					selection = 3;
					
				if(StrCompare(class, "Biology") == 0)
					selection = 4;
				
				if(StrCompare(class, "French") == 0)
					selection = 5;
				
				if(StrCompare(class, "Geology") == 0)
					selection = 6;
				
				if(StrCompare(class, "Geography") == 0)
					selection = 7;
				
				if(StrCompare(class, "History") == 0)
					selection = 8;
				
				if(StrCompare(class, "Computers") == 0)
					selection = 9;
				
				if(StrCompare(class, "Physics") == 0)
					selection = 10;
				
				if(StrCompare(class, "Chemistry") == 0)
					selection = 11;
				
				if(StrCompare(class, "Spanish") == 0)
					selection = 12;
				
				
									
				ctl = GetObjectPtr (EventEditClassPopTrigger);
				lst = GetObjectPtr (EventEditClassList);
		
				// get the correspondent text to the current selection number
					label = LstGetSelectionText (lst, selection);
				
				// the text of the trigger is now the current class name
					CtlSetLabel (ctl, label);
				
				// set the selected class to the class in the struct
					LstSetSelection (lst, selection);	
			
					
			
			}
		
			FrmDrawForm (prevForm);		
			if(year[currentmonth][currentday].used == true)	{	
		// now for the event type field
				eventFldPtr = FrmGetObjectPtr(prevForm, FrmGetObjectIndex(prevForm, EventEditTypeField));
				event= year[currentmonth][currentday].event;
				
			// get another handle to handle another situation
				eventHandle = MemHandleNew(StrLen(event) + 1);
			
			// Lock down the handle and get a pointer to the memory chunk.
				eventText = MemHandleLock(eventHandle);	
			
			// Copy the data from the record to the new memory chunk.
				StrCopy(eventText, event);
			
			// Set the field's text to the data in the new memory chunk.
				FldSetTextHandle(eventFldPtr, eventHandle);
			
			// redraw this sucker
				FldDrawField(eventFldPtr);
			
			// We don't need it anymore so we unlock the chunk... go go! you're free!
				MemHandleUnlock(eventHandle);
			}
	
	// now the event day
			
			currentdayptr = FrmGetObjectPtr(prevForm, FrmGetObjectIndex(prevForm, EventEditEventDayField));
		
		// add 1 to currentday so it displays the right date
			currentday++; 
		
		// convert it to ASCII
			currentdaystr = StrIToA(s, currentday); 	
		
		// get a new handle to handle the situation
			dayHandle = MemHandleNew(StrLen(currentdaystr) + 1);
		
		// Lock down the handle and get a pointer to the memory chunk.
			dayText = MemHandleLock(dayHandle);	
		
		// Copy the data from the record to the new memory chunk.
			StrCopy(dayText, currentdaystr);	
		
		// Set the field's text to the data in the new memory chunk.
			FldSetTextHandle(currentdayptr, dayHandle);
		
		// redraw this sucker...
			FldDrawField(currentdayptr);
		
		// Unlock the new memory chunk.
			MemHandleUnlock(dayHandle);	
			currentday--;
		
	// now for the event month
			currentmonthptr = FrmGetObjectPtr(prevForm, FrmGetObjectIndex(prevForm, EventEditEventMonthField));
		
		// add 1 to currentday so it displays the right date
			currentmonth++; 
		
		// convert it to ASCII
			currentmonthstr = StrIToA(s, currentmonth);
		
		// get another handle to handle another situation
			monthHandle = MemHandleNew(StrLen(currentmonthstr) + 1);
		
		// Lock down the handle and get a pointer to the memory chunk.
			monthText = MemHandleLock(monthHandle);	
		
		// Copy the data from the record to the new memory chunk.
			StrCopy(monthText, currentmonthstr);
		
		// Set the field's text to the data in the new memory chunk.
			FldSetTextHandle(currentmonthptr, monthHandle);
		
		// redraw this sucker
			FldDrawField(currentmonthptr);
		
		// We don't need it anymore so we unlock the chunk... go go! you're free!
			MemHandleUnlock(monthHandle);

			currentmonth--;

				
			handled = true;
			break;

		default:
			break;
		
		}
	
	return handled;
}


#pragma mark ----------------


static Boolean FindNextEvent(void) {
UInt8 day, month;
Boolean noMoreEvents = false;


	day = currentday;
	month = currentmonth;
	
	do {
		if(month == 11 && day == 30) {
			noMoreEvents = true;
			break;
		}
		if(day == 30) {
			day = 0;
			month++;
		}
		else
			day++;
		
	} while(year[month][day].used != true);
	
	if(noMoreEvents != true) {
		currentday = day;
		currentmonth = month;
	}
	
	return noMoreEvents;
}

static Boolean FindPreviousEvent(void) {
UInt8 day, month;
Boolean noMoreEvents = false;

	day = currentday;
	month = currentmonth;
	
	do {
		if(month == 0 && day == 0) {
			noMoreEvents = true;
			break;
		}
		if(day == 0) {
			day = 30;
			month--;
		}
		else
			day--;
		
	} while(year[month][day].used != true);

	if(noMoreEvents != true) {
		currentday = day;
		currentmonth = month;
	}
	
	return noMoreEvents;

}
/***********************************************************************
 *
 * FUNCTION:    LocateNextEventHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "LocateNextEvent" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean LocateNextEventHandleEvent(EventPtr eventP)
{
   Boolean handled = false;
   FormPtr frmP;
   Char *currentdaystr, *currentmonthstr, s[4];
   MemPtr	dayText, monthText, classText, eventText;
   MemHandle	monthHandle, dayHandle, classHandle, eventHandle;
   FieldType *currentdayptr, *currentmonthptr, *classFldPtr, *eventFldPtr;
   Boolean noMoreEvents;
   
 	frmP = FrmGetActiveForm();
	currentdayptr = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, LocateNextEventDayField));
	currentmonthptr = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, LocateNextEventMonthField));
	classFldPtr = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, LocateNextEventClassField));
	eventFldPtr = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, LocateNextEventTypeField));

	switch (eventP->eType) {
		case ctlSelectEvent:
			
			
			switch(eventP->data.ctlEnter.controlID) {
				case LocateNextEventOKButton:
					// dismiss dialog
						FrmReturnToForm(0);
						//GADGET DRAW - always update the gadget... we might have edited some records
						frmP = FrmGetActiveForm();
						if(FrmGetActiveFormID() == MainForm)
							GadgetDraw(frmP, Main1Gadget);
						handled = true;
						break;
				case LocateNextEventEditEventButton:
						FrmPopupForm (EventEditForm);
						
						handled = true;	
						break;
				case LocateNextEventNextButton:
							noMoreEvents = FindNextEvent();
							if(noMoreEvents == true) 
								FrmAlert(NoMoreEventsAlert);
							
							else {
							// we need to update the fields with the information of the day we're editing
								currentday++; // add 1 to currentday so it displays the right date
								currentdaystr = StrIToA(s, currentday); // convert it to ASCII	
							
							// get a new handle to handle the situation
								dayHandle = MemHandleNew(StrLen(currentdaystr) + 1);
							
							// Lock down the handle and get a pointer to the memory chunk.
								dayText = MemHandleLock(dayHandle);	
							
							// Copy the data from the record to the new memory chunk.
								StrCopy(dayText, currentdaystr);	
							
							// Set the field's text to the data in the new memory chunk.
								FldSetTextHandle(currentdayptr, dayHandle);
							
							// redraw this sucker...
								FldDrawField(currentdayptr);
							
							// Unlock the new memory chunk.
								MemHandleUnlock(dayHandle);	
								currentday--;
		
							// now for the month
			
								currentmonth++; // add 1 to currentday so it displays the right date
								currentmonthstr = StrIToA(s, currentmonth); // convert it to ASCII
							
							// resize the handle to handle another situation
								monthHandle = MemHandleNew(StrLen(currentmonthstr) + 1);
							
							// Lock down the handle and get a pointer to the memory chunk.
								monthText = MemHandleLock(monthHandle);	
							// Copy the data from the record to the new memory chunk.
								StrCopy(monthText, currentmonthstr);
							// Set the field's text to the data in the new memory chunk.
								FldSetTextHandle(currentmonthptr, monthHandle);
							// redraw this sucker
								FldDrawField(currentmonthptr);
							// We don't need it anymore so we unlock the chunk... go go! you're free!
								MemHandleUnlock(monthHandle);

								currentmonth--;
	
						// now for the class
			
							// resize the handle to handle another situation
								classHandle = MemHandleNew(StrLen(year[currentmonth][currentday].class) + 1);
		
							// Lock down the handle and get a pointer to the memory chunk.
								classText = MemHandleLock(classHandle);	
							
							// Copy the data from the record to the new memory chunk.
								StrCopy(classText, year[currentmonth][currentday].class);
							// Set the field's text to the data in the new memory chunk.
								FldSetTextHandle(classFldPtr, classHandle);
							// redraw this sucker
								FldDrawField(classFldPtr);
							// We don't need it anymore so we unlock the chunk... go go! you're free!
								MemHandleUnlock(classHandle);

	
						// now for the event type field
		
							// get another handle to handle another situation
								eventHandle = MemHandleNew(StrLen(year[currentmonth][currentday].event) + 1);
		
							// Lock down the handle and get a pointer to the memory chunk.
								eventText = MemHandleLock(eventHandle);	
		
							// Copy the data from the record to the new memory chunk.
								StrCopy(eventText, year[currentmonth][currentday].event);
		
							// Set the field's text to the data in the new memory chunk.
								FldSetTextHandle(eventFldPtr, eventHandle);
		
							// redraw this sucker
								FldDrawField(eventFldPtr);
		
							// We don't need it anymore so we unlock the chunk... go go! you're free!
								MemHandleUnlock(eventHandle);

							}
							handled = true;
						break;
				case LocateNextEventPreviousButton:
						noMoreEvents = FindPreviousEvent();
							if(noMoreEvents == true) 
								FrmAlert(NoMoreEventsAlert);
							
							else {
							// we need to update the fields with the information of the day we're editing
								currentday++; // add 1 to currentday so it displays the right date
								currentdaystr = StrIToA(s, currentday); // convert it to ASCII	
							
							// get a new handle to handle the situation
								dayHandle = MemHandleNew(StrLen(currentdaystr) + 1);
							
							// Lock down the handle and get a pointer to the memory chunk.
								dayText = MemHandleLock(dayHandle);	
							
							// Copy the data from the record to the new memory chunk.
								StrCopy(dayText, currentdaystr);	
							
							// Set the field's text to the data in the new memory chunk.
								FldSetTextHandle(currentdayptr, dayHandle);
							
							// redraw this sucker...
								FldDrawField(currentdayptr);
							
							// Unlock the new memory chunk.
								MemHandleUnlock(dayHandle);	
								currentday--;
		
							// now for the month
			
								currentmonth++; // add 1 to currentday so it displays the right date
								currentmonthstr = StrIToA(s, currentmonth); // convert it to ASCII
							
							// resize the handle to handle another situation
								monthHandle = MemHandleNew(StrLen(currentmonthstr) + 1);
							
							// Lock down the handle and get a pointer to the memory chunk.
								monthText = MemHandleLock(monthHandle);	
							// Copy the data from the record to the new memory chunk.
								StrCopy(monthText, currentmonthstr);
							// Set the field's text to the data in the new memory chunk.
								FldSetTextHandle(currentmonthptr, monthHandle);
							// redraw this sucker
								FldDrawField(currentmonthptr);
							// We don't need it anymore so we unlock the chunk... go go! you're free!
								MemHandleUnlock(monthHandle);

								currentmonth--;
	
						// now for the class
			
							// resize the handle to handle another situation
								classHandle = MemHandleNew(StrLen(year[currentmonth][currentday].class) + 1);
		
							// Lock down the handle and get a pointer to the memory chunk.
								classText = MemHandleLock(classHandle);	
							
							// Copy the data from the record to the new memory chunk.
								StrCopy(classText, year[currentmonth][currentday].class);
							// Set the field's text to the data in the new memory chunk.
								FldSetTextHandle(classFldPtr, classHandle);
							// redraw this sucker
								FldDrawField(classFldPtr);
							// We don't need it anymore so we unlock the chunk... go go! you're free!
								MemHandleUnlock(classHandle);

	
						// now for the event type field
		
							// get another handle to handle another situation
								eventHandle = MemHandleNew(StrLen(year[currentmonth][currentday].event) + 1);
		
							// Lock down the handle and get a pointer to the memory chunk.
								eventText = MemHandleLock(eventHandle);	
		
							// Copy the data from the record to the new memory chunk.
								StrCopy(eventText, year[currentmonth][currentday].event);
		
							// Set the field's text to the data in the new memory chunk.
								FldSetTextHandle(eventFldPtr, eventHandle);
		
							// redraw this sucker
								FldDrawField(eventFldPtr);
		
							// We don't need it anymore so we unlock the chunk... go go! you're free!
								MemHandleUnlock(eventHandle);

							}
							handled = true;
			}
			break;
			
		case frmOpenEvent:
			FrmDrawForm (frmP);
			currentmonth = thisMonth-1;
			currentday = today-1;
			noMoreEvents = FindNextEvent();
			if(noMoreEvents == true) 
				FrmAlert(NoMoreEventsAlert);
			
		// we need to update the fields with the information of the day we're editing
			currentday++; // add 1 to currentday so it displays the right date
			currentdaystr = StrIToA(s, currentday); // convert it to ASCII	

		// get a new handle to handle the situation
			dayHandle = MemHandleNew(StrLen(currentdaystr) + 1);

		// Lock down the handle and get a pointer to the memory chunk.
			dayText = MemHandleLock(dayHandle);	

		// Copy the data from the record to the new memory chunk.
			StrCopy(dayText, currentdaystr);	

		// Set the field's text to the data in the new memory chunk.
			FldSetTextHandle(currentdayptr, dayHandle);
	
		// redraw this sucker...
			FldDrawField(currentdayptr);
	
		// Unlock the new memory chunk.
			MemHandleUnlock(dayHandle);	
			currentday--;
		
	// now for the month

			currentmonth++; // add 1 to currentday so it displays the right date
			currentmonthstr = StrIToA(s, currentmonth); // convert it to ASCII
		
		// resize the handle to handle another situation
			monthHandle = MemHandleNew(StrLen(currentmonthstr) + 1);
	
		// Lock down the handle and get a pointer to the memory chunk.
			monthText = MemHandleLock(monthHandle);	
	
		// Copy the data from the record to the new memory chunk.
			StrCopy(monthText, currentmonthstr);
		
		// Set the field's text to the data in the new memory chunk.
			FldSetTextHandle(currentmonthptr, monthHandle);
	
		// redraw this sucker
			FldDrawField(currentmonthptr);
	
		// We don't need it anymore so we unlock the chunk... go go! you're free!
			MemHandleUnlock(monthHandle);

			currentmonth--;
	
	
	// now for the class
	
		// resize the handle to handle another situation
			classHandle = MemHandleNew(StrLen(year[currentmonth][currentday].class) + 1);
	
		// Lock down the handle and get a pointer to the memory chunk.
			classText = MemHandleLock(classHandle);	
	
		// Copy the data from the record to the new memory chunk.
			StrCopy(classText, year[currentmonth][currentday].class);
	
		// Set the field's text to the data in the new memory chunk.
			FldSetTextHandle(classFldPtr, classHandle);
	
		// redraw this sucker
			FldDrawField(classFldPtr);
		
		// We don't need it anymore so we unlock the chunk... go go! you're free!
			MemHandleUnlock(classHandle);

	
	// now for the event type field
		
		// get another handle to handle another situation
			eventHandle = MemHandleNew(StrLen(year[currentmonth][currentday].event) + 1);
		
		// Lock down the handle and get a pointer to the memory chunk.
			eventText = MemHandleLock(eventHandle);	
		
		// Copy the data from the record to the new memory chunk.
			StrCopy(eventText, year[currentmonth][currentday].event);
		
		// Set the field's text to the data in the new memory chunk.
			FldSetTextHandle(eventFldPtr, eventHandle);
		
		// redraw this sucker
			FldDrawField(eventFldPtr);
		
		// We don't need it anymore so we unlock the chunk... go go! you're free!
			MemHandleUnlock(eventHandle);
			handled = true;
			break;

		default:
			break;
		
		}
	
	return handled;
}

#pragma mark ----------------

/***********************************************************************
 *
 * FUNCTION:    MainFormInit
 *
 * DESCRIPTION: This routine initializes the MainForm form.
 *
 * PARAMETERS:  frm - pointer to the MainForm form.
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void MainFormInit(FormPtr /*frmP*/)
{
}


/***********************************************************************
 *
 * FUNCTION:    MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:  MenuID  - menu item id
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormDoCommand(UInt16 MenuID)
{
	Boolean handled = false;
	FormPtr prevForm = FrmGetActiveForm();
	UInt16 prevFormID = FrmGetActiveFormID();
	FormPtr frmP; 

 	
	switch (MenuID)
		{
		case MainOptionsAbout:
			MenuEraseStatus(0);					// Clear the menu status from the display.
			frmP = FrmInitForm (AboutForm);
			FrmSetActiveForm(frmP);
			FrmDoDialog (frmP);					// Display the About Box.

			if(prevForm) 
				FrmSetActiveForm(prevForm);
				
			FrmDeleteForm (frmP);
			handled = true;
			break;
			
		case RecordLocateNextEvent:
			MenuEraseStatus(0);					// Clear the menu status from the display
			
			FrmPopupForm (LocateNextEventForm);					
			handled = true;
			break;
			
		default:
			break;
		}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr eventP)
{
   Boolean handled = false;
   FormPtr currentform = FrmGetActiveForm();
   MemHandle h = MemHandleNew(sizeof(UInt16));
   RectangleType bounds;
	event = year[0][0].event;
	switch (eventP->eType) 
		{
		case menuEvent:
			return MainFormDoCommand(eventP->data.menu.itemID);

		case frmOpenEvent:
			if(h) {
				* (UInt16 *) MemHandleLock(h) = 1;
				MemHandleUnlock(h);
				FrmSetGadgetData(currentform, FrmGetObjectIndex(currentform, Main1Gadget), h);
			}
			MainFormInit(currentform);
			FrmDrawForm (currentform);
			if(firstDraw == true) {
				GadgetInit(currentform, Main1Gadget);
				firstDraw = false;
			}
	
			
		// draw the gadget
			GadgetDraw(currentform, Main1Gadget);
			handled = true;
			break;

	// the user taps on the screen
		case penDownEvent:
		// let's get the gadget bounds 
			FrmGetObjectBounds(currentform, FrmGetObjectIndex(currentform, Main1Gadget), &bounds);
		// we now compare the gadget area with the place where the sucker tapped
		// if it's true, send the job to the proper handle - GadgetTap(); - and tell the system that we did the job 
			if(RctPtInRectangle(eventP->screenX, eventP->screenY, &bounds)) {
				GadgetTap(currentform, Main1Gadget, eventP);
				handled = true;
			}
			break;
	
		default:
			break;
		
		}
	
	return handled;
}
#pragma mark ----------------


/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void UnPackRecord(Char *recTextPtr) {
UInt8 day, month, x, y;
Char dateTxt[2];
Char classTxt[20];
Char eventTxt[6];

	dateTxt[0] = recTextPtr[0];
	dateTxt[1] = recTextPtr[1];
	day = StrAToI(dateTxt);
	
	dateTxt [0] = recTextPtr[2]; 
	dateTxt[1] = recTextPtr[3];
	month = StrAToI(dateTxt);

	x = 0;
	y = 4;
	while(recTextPtr[y] != '-') {
		classTxt[x] = recTextPtr[y];
		x++;
		y++;
	}
	classTxt[x] = '\0';
	StrCopy(year[month-1][day-1].class, classTxt);
	
	x = 0;
	for(y+=1; y <= (StrLen(recTextPtr) -1); y++) {
		eventTxt[x] = recTextPtr[y];
		x++;
	}
	eventTxt[x] = '\0';
	StrCopy(year[month-1][day-1].event, eventTxt);
	year[month-1][day-1].used = true;
	
}



Err static PackAndSaveRecords(void) {
Err error;
DmOpenRef database;
Char recText[30];
UInt8 day, month;
Char *currentdaystr, *currentmonthstr, s[4];
MemHandle textHandle;
Char *textPtr;
UInt16 index = 0;
LocalID oldDBid;
	
	oldDBid = DmFindDatabase(0, databaseName);
	error = DmDeleteDatabase(0, oldDBid);
	error = DmCreateDatabase(0, databaseName, appFileCreator, ClassDBType, false);
		
	if (error)
		return error;
	
	database = DmOpenDatabaseByTypeCreator(ClassDBType, appFileCreator, dmModeReadWrite);
	if(!database)
		return DmGetLastErr();
		
	// delete the records from the db and read the records from the struct and put them in the database
	else {	
		for(month = 0; month <= 11; month++) {
			for(day = 0; day <= 30; day++) {
				if(year[month][day].used == true) {

					currentday = day + 1; // add 1 to currentday so it displays the right date
					currentdaystr = StrIToA(s, currentday);
					
					if(StrLen(currentdaystr) < 2) {
						StrCopy(recText, "0");
						StrCat(recText, currentdaystr);
					}
					else
						StrCopy(recText, currentdaystr);

					
					currentmonth = month + 1; // add 1 to currentday so it displays the right date
					currentmonthstr = StrIToA(s, currentmonth);

					if(StrLen(currentmonthstr) < 2)
						 StrCat(recText, "0");
					StrCat(recText, currentmonthstr);
					StrCat(recText, year[month][day].class);
					StrCat(recText, "-");
					StrCat(recText, year[month][day].event);
		
					// Create a new first record in the database and get a handle to the record.
					textHandle = DmNewRecord(database, &index, StrLen(recText) + 1);
					if (textHandle == NULL)
 					     return dmErrMemError;
	
					// Lock down the handle and get a pointer to the new record.
					textPtr = MemHandleLock(textHandle); 
	
					// Write the field's text to the database record.
					DmWrite(textPtr, 0, recText, StrLen(recText) + 1);

					// Unlock the new record.
					MemHandleUnlock(textHandle);
			
					// Release the record to the database manager.  The true value indicates that
					// the record contains "dirty" data.  Release Record will set the record's 
					// dirty flag and update the database modification count.
					DmReleaseRecord(database, index, true);
				}
			}
		}
	}
	
	error = DmCloseDatabase(database);
	return error;
}
#pragma mark ----------------


/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent(EventPtr eventP)
{
	UInt16 formId;
	FormPtr frmP;

	if (eventP->eType == frmLoadEvent)
		{
		// Load the form resource.
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId)
			{
			case MainForm:
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;
			case LocateNextEventForm:
				FrmSetEventHandler(frmP, LocateNextEventHandleEvent);
				break;
			case EventEditForm:
				FrmSetEventHandler(frmP, EventEditHandleEvent);
				break;
			case AboutForm:
				FrmSetEventHandler(frmP, AboutFormHandleEvent);
				break;
			default:
				ErrFatalDisplay("Invalid Form Load Event");
				break;

			}
		return true;
		}
	
	return false;
}


/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
	UInt16 error;
	EventType event;

	do {
		EvtGetEvent(&event, evtWaitForever);

		if (! SysHandleEvent(&event))
			if (! MenuHandleEvent(0, &event, &error))
				if (! AppHandleEvent(&event))
					FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}

#pragma mark ----------------

/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value 0 if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{

//    UInt16 prefsSize;
	DmOpenRef database;
	Char *recText;
	MemHandle currentRecHandle;
	UInt16 currentRecord;
	Err error;
	UInt8 day,month;
	UInt16 numRecords;
	DateTimeType dateTime;
	
	for(month = 0; month <= 11; month++) {
		for(day = 0; day <= 30; day++) {
			StrCopy(year[month][day].class, "none");
		}		
	}	
	database = DmOpenDatabaseByTypeCreator(ClassDBType, appFileCreator, dmModeReadWrite);
	if(!database) {
		error = DmCreateDatabase(0, databaseName, appFileCreator, ClassDBType, false);
		
		if (error)
			return error;
		
		database = DmOpenDatabaseByTypeCreator(ClassDBType, appFileCreator, dmModeReadWrite);
		
		if(!database)
			return DmGetLastErr();
	}
	// read the records stored in the database and put them in the struct
	else {
		numRecords = DmNumRecordsInCategory(database, dmAllCategories);
		if(numRecords > 0) {
			for(currentRecord=0; currentRecord <= numRecords -1; currentRecord++) {
				currentRecHandle = DmGetRecord(database,currentRecord);
				if(currentRecHandle) {
					recText = MemHandleLock(currentRecHandle);
					UnPackRecord(recText);
					MemHandleUnlock(currentRecHandle);
					DmReleaseRecord(database, currentRecord, false);
				}		 
			}
		}
	}
	
	error = DmCloseDatabase(database);
	if(error)
		return error;
	
	
	// set todays date	
		TimSecondsToDateTime (TimGetSeconds (), &dateTime);
		today = dateTime.day;
		thisMonth = dateTime.month;
		thisYear = dateTime.year;
   return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{
/*   StarterPreferenceType prefs;

	// Write the saved preferences / saved-state information.  This data 
	// will be backed up during a HotSync.
	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		&prefs, sizeof (prefs), true);*/
		
	// Pack and save all the records
	PackAndSaveRecords();
	/*if(err)
		dialog("the database blahblahablaha") */
	// Close all the open forms.
	FrmCloseAllForms ();
}


/***********************************************************************
 *
 * FUNCTION:    StarterPalmMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 *
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static UInt32 StarterPalmMain(UInt16 cmd, MemPtr /*cmdPBP*/, UInt16 launchFlags)
{
	Err error;

	error = RomVersionCompatible (ourMinVersion, launchFlags);
	if (error) return (error);

	switch (cmd)
		{
		case sysAppLaunchCmdNormalLaunch:
			error = AppStart();
			if (error) 
				return error;
				
			FrmGotoForm(MainForm);
			AppEventLoop();
		// OH MY GOD WE'RE GOIN' TO GET KILLED! #$%"#&#
			AppStop();
			break;

		default:
			break;

		}
	
	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    return StarterPalmMain(cmd, cmdPBP, launchFlags);
}
