/******************************************************************************
 *
 * ClassDB - A simple school calendar 
 * Copyright (C) 2001 Ricardo Cabral - nightdog@users.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *****************************************************************************/


#include <PalmOS.h>
#include "ClassDBrsc.h"
#include "Prototypes.h"

extern EVENTS year[12][31];
extern UInt8 currentday, currentmonth;
extern Int16 today, thisMonth, thisYear;


// this function checks a given date to see if it is valid
static Boolean forbiddenDay(Int16 month, Int16 day) {
Boolean forbidden = false;
	if(DaysInMonth(month + 1, thisYear) < day)
		forbidden = true;
return forbidden;
}


void GadgetInit(FormPtr frm, UInt16 gadgetID) {
UInt16 fromx, fromy,cellWidth, cellHeight;
UInt8 day, month;
UInt16 gadgetIndex = FrmGetObjectIndex (frm, gadgetID);
RectangleType bounds;


MemHandle data = FrmGetGadgetData (frm, gadgetIndex);

	if(data) {
		FrmGetObjectBounds(frm, gadgetIndex, &bounds);
		
		cellWidth= bounds.extent.x - FntCharWidth('M');
		cellWidth = cellWidth/12;
		cellHeight = bounds.extent.y - FntCharHeight();
		cellHeight = cellHeight/31;
		
		
		fromx= bounds.topLeft.x + FntCharWidth('M'); 
		for(month=0; month <=11; month++) {
			fromy= bounds.topLeft.y + FntCharHeight();
			for(day = 0; day <= 30; day++) {
				if(forbiddenDay(month, day) == true)
					year[month][day].validDate = false;
				else {
					
					year[month][day].dayCell.topLeft.x = fromx-1;
					year[month][day].dayCell.topLeft.y = fromy +1;
					year[month][day].dayCell.extent.x = cellWidth-1;
					year[month][day].dayCell.extent.y = cellHeight-1;
				
				}
				fromy += cellHeight;		
			}
			fromx +=cellWidth;
		}	
	}	
}



void GadgetDraw(FormPtr frm, UInt16 gadgetID) {
RectangleType bounds;
UInt16 fromx, fromy, tox, toy;
UInt16 gadgetIndex = FrmGetObjectIndex (frm, gadgetID);
UInt16 i;
Char *monthInitialsPtr;
UInt16 cellWidth, cellHeight;
UInt8 times,day,month;
MemHandle monthInitialsHandle;
	
	//lets get the bounds of the devil gadget
	FrmGetObjectBounds(frm, gadgetIndex, &bounds);
	// wipe it out
	WinEraseRectangle(&bounds, 0);

	
	// draw " Days " vertically, exactly in the middle of the table
	fromx = bounds.topLeft.x;
	fromy = bounds.extent.x / 2 - (FntCharsWidth("Days", StrLen("Days")) / 2);
	WinDrawChar('D', fromx,fromy);
	fromy += (FntCharHeight() + 1);
	WinDrawChar('a', fromx,fromy);
	fromy += (FntCharHeight() + 1);
	WinDrawChar('y', fromx,fromy);
	fromy += (FntCharHeight() + 1);
	WinDrawChar('s', fromx,fromy);

	cellWidth = bounds.extent.x - FntCharWidth('M');
	cellWidth = cellWidth/12;
	cellHeight = bounds.extent.y - FntCharHeight();
	cellHeight = cellHeight/31;

	
// draw The Months horizontally, at the top of the table
	monthInitialsHandle = DmGetResource (strRsc, monthInitialsString);
	monthInitialsPtr = MemHandleLock (monthInitialsHandle);
	fromx = bounds.topLeft.x+ FntCharWidth('M');
	
	for(i=0; i <= StrLen(monthInitialsPtr) - 1;i++) {
		fromy = bounds.topLeft.y;
		WinDrawChar(monthInitialsPtr[i], fromx, fromy);
		fromx += cellWidth;
	}
	MemHandleUnlock(monthInitialsHandle);
	
// draw the horizontal lines... 32 lines so you can make 31 columns for each day	
	fromx = bounds.topLeft.x + FntCharWidth('D'); 
	tox= fromx + cellWidth*12;
	i=bounds.topLeft.y + FntCharHeight(); // where we start to draw the lines
	
	for(times = 1; times <= 32;times++) {
		fromy = i;
		toy = fromy;
		WinDrawLine(fromx, fromy, tox, toy); // here we actually *DRAW* it 
	// the next line will be parallel to this one with a space of cellHeight between them
		i += cellHeight; 
	}
	
	
// draw the vertical lines... 13 lines so you can make 12 columns for each month
// same mechanism in here with the exception of the toy being the end of the horizontal lines, so they can connect
			
	fromy = bounds.topLeft.y + FntCharHeight();
	toy = i - cellHeight;	
	i = bounds.topLeft.x + FntCharWidth('D');

	for(times = 1; times <= 13; times++) {
		fromx = i;
		tox = fromx;
		WinDrawLine(fromx, fromy, tox, toy);
		i += cellWidth;
	}
	
	// draw today
		fromx = year[thisMonth -1][today - 1].dayCell.topLeft.x;
		tox = fromx + cellWidth -2;
		fromy = year[thisMonth -1][today - 1].dayCell.topLeft.y + (cellHeight/2) -1;
		toy = fromy;
		WinDrawLine(fromx,fromy,tox,toy);
		WinInvertRectangle(&year[thisMonth -1][today-1].dayCell, 0);
	
	// draw forbidden days in grey - this is not done 
	
	// draw days that have events
		for(month = 0; month <= 11; month++) {
			for(day = 0; day <= 30; day++) {
				if(year[month][day].used == true) {
					fromx = year[month][day].dayCell.topLeft.x;
					fromy = year[month][day].dayCell.topLeft.y-1;
					tox = fromx +year[month][day].dayCell.extent.x -1;
					toy = fromy + year[month][day].dayCell.extent.y+1;
					WinDrawLine(fromx, fromy, tox, toy);
				
					WinDrawLine(fromx, toy, tox, fromy);
				}
			}
		}
}


void GadgetTap(FormPtr frm, UInt16 gadgetID, EventPtr event) {
MemHandle data = FrmGetGadgetData (frm, FrmGetObjectIndex(frm, gadgetID));
Boolean wasInBounds = true;
RectangleType bounds, gridBounds;
UInt8 day, month;
Int16 x,y;
Boolean penDown;
Boolean nowInCell; 
UInt16 cellWidth, cellHeight;
Boolean isThisTheCell = false;


	FrmGetObjectBounds(frm, FrmGetObjectIndex(frm, gadgetID), &bounds);

	cellWidth = bounds.extent.x - FntCharWidth('M');
	cellWidth = cellWidth/12;
	cellHeight = bounds.extent.y - FntCharHeight();
	cellHeight = cellHeight/31;
		
	// let's set the limits of the grid so we can only deal with taps inside it
	gridBounds.topLeft.x = bounds.topLeft.x + FntCharWidth('D');
	gridBounds.topLeft.y = bounds.topLeft.y + FntCharHeight();
	gridBounds.extent.x = 12 * cellWidth;
	gridBounds.extent.y = 31 * cellHeight;
		
	if(RctPtInRectangle(event->screenX, event->screenY, &gridBounds)) {

		month = day = 0;
		do {
			if(day == 30 && month == 11)
				break;
			isThisTheCell= RctPtInRectangle(event->screenX, event->screenY, &year[month][day].dayCell);
			if(day==30) {
				day = 0;
				month++;
			}
			else
				day++;		
		} while(isThisTheCell != true);
		if(isThisTheCell == true) {
		if(year[month][day].validDate == false) {
			WinInvertRectangle(&year[month][day].dayCell, 1);
			do {	  
				PenGetPoint(&x, &y, &penDown);
				nowInCell = RctPtInRectangle(x,y,&year[month][day].dayCell);
				if(nowInCell == true) {
					WinInvertRectangle(&year[month][day].dayCell, 0);		
				}
			} while(penDown);
			currentday = day;
			currentmonth = month;
			FrmPopupForm (EventEditForm);
		}	
		}
	}
}


